/**
 * Class SnakePart
 * The parent of all snake parts.
 * Handles the creation and destruction of SnakeParts.
 */
public abstract class SnakePart implements IContent{

	// Number of SnakePart objects
	public static int idnum = 0;

	// ID of an exact SnakePart object
	protected int id;
 
	// Field
	//protected Field field;

	//Reference to the FieldCell containig this SnakePart object.
	protected FieldCell cell;

	//Reference to this SnakePart's proprietor.
	public Snake snake;

	// Reference to the body element following this one.
	protected SnakeBody nextBodyElement;

	// Reference to the body element following this one.
	protected SnakePart prevBodyElement;

	// Indicates whether this body element contains a stone or not.
	protected boolean hasStone;

	// Time till stone rolls;
	protected int timeTillRoll;
	
	// Type of IContent
	public int type;

	// Contains the direction in which the snake is heading.
	protected int direction = Enum.RIGHT;



/**
 * Class constructor.
 */
	public SnakePart(){
	}



/**
 * Handles the destruction of the Snake.
 */
	public void destroy(){
	}



/**
 * Returns the position of the Snake's bodypart.
 */
	public FieldCell getPosition(){
		return cell;
	}



/**
 * Sets position.
 */
	public void setPosition(FieldCell pos){
		cell = pos;
	}



/**
 * Gets owner.
 */
	public Snake getSnake(){
		return snake;
	}



/**
 * Sets owner.
 */
	public void setSnake(Snake s){
		snake = s;
	}



/**
 * Returns the next element of the Snake's body.
 */
	public boolean hasNext(){
		if (nextBodyElement != null)
			return true;
		return false;
	}




/**
 * Returns the next element of the Snake's body.
 */
	public SnakeBody getNext(){
		return nextBodyElement;
	}



/**
 * Set next bodypart.
 */
	public void setNext(SnakeBody next){
		nextBodyElement = next;
	}



/**
 * Returns the previous element of the Snake's body.
 */
	public SnakePart getPrev(){
		return prevBodyElement;
	}



/**
 * Set previous bodypart.
 */
	public void setPrev(SnakePart prev){
		prevBodyElement = prev;
	}



/**
 * Initializes the direction of movement.
 *
 * @param dir Direction of movement.
 */
	public void initDirection(int dir){
		if (prevBodyElement == null)
			direction = dir;
		else{
			for (int i=0; i<Enum.NUMNEIGHBOURS; i++){
				if (cell.getNeighbour(i).getContent().equals(prevBodyElement.getPosition().getContent()))
					direction = i;
			}
		}
		if (nextBodyElement != null){
			nextBodyElement.initDirection(direction);
		}
	}



/**
 * Returns direction of movement.
 */
	public int getDirection(){
		return direction;
	}



/**
 * To set direction of movement by the player.
 *
 * @param dir Direction of movement.
 */
	public void setDirection(int dir){
		direction = dir;
	}



/**
 * Sets initial length
 */
	public SnakePart[] initLength(int num, int length, SnakePart[] parts, FieldCell[] pos, SnakePart prev){
		this.setPosition(pos[num]);
		parts[num] = this;
		this.setPrev(prev);
		length--;
		if (length > 0){
			if (nextBodyElement == null)
				nextBodyElement = new SnakeBody(snake);
			num++;
			parts = nextBodyElement.initLength(num, length, parts, pos, this);
		}
		else
			nextBodyElement = null;
		return parts;
	}



/**
 * Returns true if the bodypart contains stone,
 * false if not.
 */
	public boolean getStoneStatus(){
		return hasStone;
	}



/**
 * Returns the type of IContent
 */
	public int getType(){
		return type;
	}



/**
 * Returns id.
 */
	public int getID(){
		return id;
	}



/**
 * Handles the passing of stones to the next SnakeBody
 * instance.
 *
 */
	public void rollStone(){
		// Lowers the time left till stone rolls
		if (timeTillRoll > 0)
			timeTillRoll--;

		// Passes Stone to next bodypart, if possible
		if (nextBodyElement != null){
			nextBodyElement.rollStone();
			if (hasStone && timeTillRoll == 0){
				if (nextBodyElement.receiveStone()){
					hasStone = false;
				}
			}
		}
	}



/**
 * Handles the receiving of stones from previous elements.
 *
 * @return reveive true, if received stone, false if not
 */
	public boolean receiveStone(){
		boolean r = true;
		// Snake's head receives stone.
		if (type == Enum.SNAKEHEAD){
			// Head receives a stone while having already
			// => the snake dies.
			if (hasStone || SnakeState.isFullOfStones(snake.getState())){
				snake.die();
				r = false;
			}
			// Head receives a stone while not having yet
			else{
				hasStone = true;
				timeTillRoll = Enum.STONEROLLTIME;
				r = true;
			}
		}
		// Snake's body receives a stone
		else{
			// Body receives a stone while having alredy
			if (hasStone)
				r = false;
			// Body receives a stone while not having yet
			else{
				hasStone = true;
				timeTillRoll = Enum.STONEROLLTIME;
				r = true;
			}
		}
		return r;
	}



/**
 * Helps to check if snake is in FULLOFSTONE state
 */
	public boolean hasStone(int n){
		if (n == 0)
			return hasStone;
		else{
			n--;
			if (nextBodyElement != null)
				return nextBodyElement.hasStone(n);
		}
		return false;
	}



/**
 * Reacts to the collision, calls back the Snake 
 * that collided with this one.
 *
 * @param from The Snake's reference that collided with this SnakeHead.
 */
	public void reactToCollision(Snake from){

		// Calls back the colliding Snake and tells that
		// it collided with another snake full of stones
		// or a bodypart containing stone.

		if (SnakeState.isFullOfStones(snake.getState()) || hasStone){
			from.collideWithStonedSnake(snake);
			return;
		}

		// If colliding snake is berserked, this snake
		// is biten off from here.
		if (SnakeState.isBerserked(from.getState())){
			snake.bitenOff(this);
			return;
		}

		// Calls back the colliding Snake and tells that
		// it collided with another snake in berserked state.
		if (SnakeState.isBerserked(snake.getState())){
			from.collideWithBerserkedSnake(snake);
		}

		// Callback the colliding Snake and tells that
		// it collided with another snake in normal state.
		else{
			 from.collideWithNormalSnake(snake);
		}
	}



}

